from django.test import TestCase
from rest_framework.test import APIRequestFactory

from ovp_core.schemas import OVPSchemaGenerator

# Just import to test execution
from ovp_core import apps
from ovp_core import urls
from ovp_core import serializers

class TestSchemaGenerator(TestCase):
  def test_schema_generation(self):
    """Assert that OVPSchemaGenerator executes without raising exceptions"""
    factory = APIRequestFactory()
    request = factory.post('/')

    generator = OVPSchemaGenerator()
    schema = generator.get_schema(request=request)



from ovp_core import models

from rest_framework import serializers

class GoogleAddressSerializer(serializers.ModelSerializer):
  class Meta:
    model = models.GoogleAddress
    fields = ['typed_address', 'typed_address2', 'address_line', 'city_state']
    read_only_fields = ['address_line', 'city_state']



from django.db import models

class Skill(models.Model):
  name = models.CharField('name', max_length=100)

  def __str__(self):
    return self.name

  class Meta:
    app_label = 'ovp_core'
    verbose_name = 'skill'



from rest_framework import schemas

# We don't test this as it's tested on drf framework
class OVPSchemaGenerator(schemas.SchemaGenerator): #pragma: no cover
  def get_schema(self, request=None):
      if self.endpoints is None:
          self.endpoints = self.get_api_endpoints(self.patterns)

      links = []
      for path, method, category, action, callback in self.endpoints:
          view = callback.cls()
          for attr, val in getattr(callback, 'initkwargs', {}).items():
              setattr(view, attr, val)
          view.args = ()
          view.kwargs = {}
          view.format_kwarg = None

          actions = getattr(callback, 'actions', None)
          if actions is not None:
              if method == 'OPTIONS':
                  view.action = 'metadata'
              else:
                  view.action = actions.get(method.lower())

          if request is not None:
              view.request = schemas.clone_request(request, method)
              try:
                  view.check_permissions(view.request)
              except schemas.exceptions.APIException:
                  continue
          else:
              view.request = None

          link = self.get_link(path, method, callback, view)
          links.append((category, action, link, method))

      if not links:
          return None

      # Generate the schema content structure, eg:
      # {'users': {'list': Link()}}
      content = {}
      for category, action, link, method in links:
          if category is None:
              content[action] = link
          elif category in content:
              if action in content[category]:
                content[category]["{}-{}".format(action, method)] = link
              else:
                content[category][action] = link
          else:
              content[category] = {action: link}

      # Return the schema document.
      return schemas.coreapi.Document(title=self.title, content=content, url=self.url)



#!/usr/bin/env python3
import glob
import os
import sys

import django
from django.conf import settings
from django.core.management import execute_from_command_line


BASE_DIR = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(BASE_DIR, '../..')))

# Unfortunately, apps can not be installed via ``modify_settings``
# decorator, because it would miss the database setup.
CUSTOM_INSTALLED_APPS = (
    'ovp_core',
    'django.contrib.admin',
)

ALWAYS_INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
)

ALWAYS_MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

REST_FRAMEWORK = {
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
    'PAGE_SIZE': 20,
    'PAGINATE_BY_PARAM': 'page_size',
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework.authentication.BasicAuthentication',
    )
}

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


settings.configure(
    SECRET_KEY="django_tests_secret_key",
    DEBUG=False,
    TEMPLATE_DEBUG=False,
    ALLOWED_HOSTS=[],
    INSTALLED_APPS=ALWAYS_INSTALLED_APPS + CUSTOM_INSTALLED_APPS,
    MIDDLEWARE_CLASSES=ALWAYS_MIDDLEWARE_CLASSES,
    ROOT_URLCONF='ovp_core.urls',
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
        }
    },
    LANGUAGE_CODE='en-us',
    TIME_ZONE='UTC',
    USE_I18N=True,
    USE_L10N=True,
    USE_TZ=True,
    STATIC_URL='/static/',
    # Use a fast hasher to speed up tests.
    PASSWORD_HASHERS=(
        'django.contrib.auth.hashers.MD5PasswordHasher',
    ),
    FIXTURE_DIRS=glob.glob(BASE_DIR + '/' + '*/fixtures/'),
    EMAIL_BACKEND='django.core.mail.backends.locmem.EmailBackend',
    TEMPLATES = [
        {
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': [os.path.abspath(os.path.join(BASE_DIR, '../../templates'))],
            'APP_DIRS': True,
            'OPTIONS': {
                'context_processors': [
                    'django.template.context_processors.debug',
                    'django.template.context_processors.request',
                    'django.contrib.auth.context_processors.auth',
                    'django.contrib.messages.context_processors.messages',
                ],
            },
        },
    ],
    DEFAULT_SEND_EMAIL='sync',
    REST_FRAMEWORK=REST_FRAMEWORK,
    AUTH_PASSWORD_VALIDATORS=AUTH_PASSWORD_VALIDATORS
)

django.setup()
args = [sys.argv[0], 'test']
# Current module (``tests``) and its submodules.
test_cases = '.'

# Allow accessing test options from the command line.
offset = 1
try:
    sys.argv[1]
except IndexError:
    pass
else: #pragma: no cover
    option = sys.argv[1].startswith('-')
    if not option:
        test_cases = sys.argv[1]
        offset = 2

args.append(test_cases)
# ``verbosity`` can be overwritten from command line.
args.append('--verbosity=2')
args.extend(sys.argv[offset:])

execute_from_command_line(args)



from django.db import models
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q, Count
from django.db.models.signals import post_save
from django.dispatch import receiver

import os
import requests

class AddressComponentType(models.Model):
  name = models.CharField(max_length=100)

  def __str__(self):
    return self.name

class AddressComponent(models.Model):
  long_name = models.CharField(max_length=400)
  short_name = models.CharField(max_length=400)
  types = models.ManyToManyField(AddressComponentType)

  def __str__(self):
    return self.long_name

class GoogleRegion(models.Model):
  region_name = models.CharField(max_length=400)
  filter_by = models.CharField(max_length=400)

class GoogleAddress(models.Model):
  typed_address = models.CharField(max_length=400, blank=True, null=True)
  typed_address2 = models.CharField(max_length=400, blank=True, null=True)
  address_line = models.CharField(max_length=400, blank=True, null=True)
  city_state = models.CharField(max_length=400, blank=True, null=True)
  lat = models.FloatField('lat', blank=True, null=True)
  lng = models.FloatField('lng', blank=True, null=True)
  address_components = models.ManyToManyField(AddressComponent)

  def get_city_state(self):
    state = self.address_components.filter(types__name='administrative_area_level_1')
    county = self.address_components.filter(types__name='administrative_area_level_2')
    sublocality = self.address_components.filter(types__name='sublocality')
    locality = self.address_components.filter(types__name='locality')

    s = u""
    if locality.count():
      s += u"{}, ".format(locality[0].long_name)
    elif county.count():
      s += u"{}, ".format(county[0].long_name)
    elif sublocality.count():
      s += u"{}, ".format(sublocality[0].long_name)

    if state.count():
      s += state[0].short_name

    return s

  def get_address(self):
    # Components types for address
    address = {'route': '', 'sublocality_level_1': '', 'administrative_area_level_2': '', 'administrative_area_level_1': '', 'country': '', 'street_number': ''}

    # Fill address dict
    for component in self.address_components.all():
      for component_type in component.types.all():
        if component_type.name in address:
          address[component_type.name] = {'short_name': component.short_name, 'long_name': component.long_name}

    # Build address string
    string_address = ''
    if 'route' in address and isinstance(address['route'], dict):
      string_address += '{}, '.format(address['route']['long_name'])
    if 'route' in address and isinstance(address['street_number'], dict):
      string_address += '{}, '.format(address['street_number']['long_name'])
    if 'sublocality_level_1' in address and isinstance(address['sublocality_level_1'], dict):
      string_address += '{}, '.format(address['sublocality_level_1']['long_name'])
    if 'administrative_area_level_2' in address and isinstance(address['administrative_area_level_2'], dict):
      string_address += '{}, '.format(address['administrative_area_level_2']['long_name'])
    if 'administrative_area_level_1' in address and isinstance(address['administrative_area_level_1'], dict):
      string_address += '{}, '.format(address['administrative_area_level_1']['short_name'])
    if 'country' in address and isinstance(address['country'], dict):
      string_address += '{}, '.format(address['country']['long_name'])

    string_address = string_address.strip().strip(',')

    return string_address


  def __str__(self):
    if self.address_line:
      return self.address_line
    return ""


@receiver(post_save, sender=GoogleAddress)
def update_address(sender, instance, **kwargs):
  # Query lat and lng at google
  #r = requests.get('https://maps.googleapis.com/maps/api/geocode/json?latlng={}%20{}'.format(instance.lat, instance.lng))
  addressline = instance.typed_address

  url = 'https://maps.googleapis.com/maps/api/geocode/json?language=pt_BR&address={}'.format(addressline)

  key = os.environ.get('GOOGLE_MAPS_KEY', None)
  if key: #pragma: no cover
    url = '{}&key={}'.format(url, key)

  r = requests.get(url)
  data = r.json()

  # Iterate through address components
  instance.address_components.clear()
  if len(data['results']) > 0:
    for component in data['results'][0]['address_components']:
    # TODO: Do not work only with first result
      # Look for component with same name and type
      ac = AddressComponent.objects.annotate(count=Count('types')).filter(long_name=component['long_name'], short_name=component['short_name'])
      for component_type in component['types']:
        ac = ac.filter(types__name=component_type)
      ac = ac.filter(count=len(component['types']))

      if not ac.count():
      # Component not found, creating
        ac = AddressComponent(long_name=component['long_name'], short_name=component['short_name'])
        ac.save()
      else:
        ac = ac.first()
        ac.types.clear()
        ac.save()


      # Add types for component
      for ctype in component['types']:
        try:
          at = AddressComponentType.objects.get(name=ctype)
        except ObjectDoesNotExist:
          at = AddressComponentType(name=ctype)
          at.save()
        ac.types.add(at)

      instance.address_components.add(ac)

    try:
      if data['results'][0]['geometry']:
        GoogleAddress.objects.filter(pk=instance.pk).update(lat=data['results'][0]['geometry']['location']['lat'], lng=data['results'][0]['geometry']['location']['lng'])
    except: #pragma: no cover
      pass

    # Using update to avoid post_save signal
    GoogleAddress.objects.filter(pk=instance.pk).update(address_line=instance.get_address(), city_state=instance.get_city_state())
    #update_searchindex(instance)



from django.test import TestCase
from django.core import mail

import ovp_core.emails

class TestBaseMail(TestCase):
  def test_email_trigger(self):
    """Assert that email is sent to outbox"""
    bm = ovp_core.emails.BaseMail('a@b.c')
    bm.sendEmail('base', '', {})
    self.assertTrue(len(mail.outbox) > 0)

  def test_async_email_trigger(self):
    """Assert that async emails are sent to outbox"""
    bm = ovp_core.emails.BaseMail('a@b.c', async_mail=True)
    bm.sendEmail('base', '', {}).join()
    self.assertTrue(len(mail.outbox) > 0)



from django.test import TestCase

# Just import to test execution
from ovp_core import validators

class TestAddressValidator(TestCase):
  def test_validation_functionn(self):
    """Assert that address_validate doesn't raise exception on valid address"""
    validators.address_validate({'typed_address': 'R. Abc'})



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-25 01:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ovp_core', '0002_auto_20161019_1557'),
    ]

    operations = [
        migrations.CreateModel(
            name='Cause',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='name')),
            ],
            options={
                'verbose_name': 'cause',
                'verbose_name_plural': 'causes',
            },
        ),
        migrations.CreateModel(
            name='Skill',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='name')),
            ],
            options={
                'verbose_name': 'skill',
            },
        ),
    ]
